import os
from cryptography.fernet import Fernet


homedir_path = os.getenv('HOME')
folder_path = homedir_path + '/.ezissue/'
hub_path = folder_path + 'hub'
lab_path = folder_path + 'lab'


def create_secure_key():
    """
    Creates a file with the user's secure key for encription purposes.
    """
    check_folder_structure(folder_path)
    path = folder_path + 'key.key'

    if os.path.isfile(path):        # this can be bugged
        return False
    else:
        key = Fernet.generate_key()
        file = open(path, 'wb+')
        file.write(key)
        file.close()
        return True


def get_key():
    """
    Gets the user's encription key from filesystem.
    """
    file = open(folder_path + 'key.key', 'rb')
    k = file.readline()
    return k


def check_folder_structure(full_path):
    """
    Creates the HOMEDIR folders if they do not exists. If they exists, it does
    nothing.
    """
    if not os.path.exists(full_path):
        os.makedirs(full_path)


def encrypt(line):
    """
    Recieves a line and encrypts it.
    """
    k = get_key()
    f = Fernet(k)
    return f.encrypt(line)


def decrypt_token(encrypted_line):
    """
    Recieves a encrypted line and decrypts it.
    """
    k = get_key()
    f = Fernet(k)
    return f.decrypt(encrypted_line)


def write_tokens(github_key, gitlab_key):
    """
    Writes user's github and gitlab access keys as encrypted content on a file.
    Returns if the operation was 'bueno' or 'no bueno'.
    """
    fp = hub_path
    fp2 = lab_path

    check_folder_structure(folder_path)

    enc_gh = encrypt(github_key.encode())
    enc_gl = encrypt(gitlab_key.encode())

    try:
        file = open(fp, "wb+")
        file2 = open(fp2, "wb+")
        file.write(enc_gh)
        file2.write(enc_gl)
    finally:
        file.close()
        file2.close()

    return (os.path.isfile(fp) and os.path.isfile(fp2))


def get_token(repo_manager):
    """
    Gets the user's access token from the filesystem given the repo manager
    he's trying to access.
    """
    enc = ''
    token = ''
    file = ''

    if repo_manager == 'github':
        file = open(hub_path, 'rb')
    elif repo_manager == 'gitlab':
        file = open(lab_path, 'rb')
    # else:
        # it would be nice to raise an exception here btw

    enc = file.read()
    token = decrypt_token(enc)
    return token.decode()



import ezissue.ezissue as ezissue


def test_md_table_row_to_array():
    md = '| get to the choppa | asdf | jkl |'
    array = ezissue.md_table_row_to_array(md)

    if not array == ['get to the choppa', 'asdf', 'jkl']:
        raise AssertionError()


def test_add_md_checkbox():
    arg = "asdf;asdf;adsf;asdf"
    resp = ezissue.add_md_checkbox(arg)
   
    if not (resp == '- [ ] asdf\n- [ ] asdf\n- [ ] adsf\n- [ ] asdf\n'):
        raise AssertionError()


def test_format_description():
    desc = "lorem ipsum dolor sit amet"
    a = ezissue.format_description(desc)

    if not (
        a == '**Issue description:**\n---\n\nlorem ipsum dolor sit amet\n\n'):
        raise AssertionError()


def test_add_prefix_to_title():
    titles = ["make america great again", "make stuff work"]
    formattedUS = []
    formattedTS = []

    for idx, title in enumerate(titles):
        formattedUS.append(
            ezissue.add_prefix_to_title(title, idx+1, 'US', '', True))
        formattedTS.append(
            ezissue.add_prefix_to_title(title, idx+1, 'TS', '', True))
    for idx, title in enumerate(titles):
        formattedUS.append(
            ezissue.add_prefix_to_title(title, idx+1, 'US', 'M', False))
        formattedTS.append(
            ezissue.add_prefix_to_title(title, idx+1, 'TS', 'M', False))
    for idx, title in enumerate(titles):
        formattedUS.append(
            ezissue.add_prefix_to_title(title, idx+1, '', '', False))
        formattedTS.append(
            ezissue.add_prefix_to_title(title, idx+1, '', '', False))

    if not (formattedUS[0] == 'US1 Make america great again' and formattedTS[0] == 'TS1 Make america great again'):
        raise AssertionError()
    
    if not (formattedUS[1] == 'US2 Make stuff work' and formattedTS[1] == 'TS2 Make stuff work'):
        raise AssertionError()

    if not (formattedUS[2] == 'USM Make america great again' and formattedTS[2] == 'TSM Make america great again'):
        raise AssertionError()
    
    if not (formattedUS[3] == 'USM Make stuff work' and formattedTS[3] == 'TSM Make stuff work'):
        raise AssertionError()

    if not (formattedUS[4] == ' Make america great again' and formattedTS[4] == ' Make america great again'):
        raise AssertionError()
   
    if not (formattedUS[5] == ' Make stuff work' and formattedTS[5] == ' Make stuff work'):
        raise AssertionError()


def test_create_github_url():
    url = ezissue.create_github_url('commit-helper', 'andre-filho')
    expected = "https://api.github.com/repos/andre-filho/commit-helper/issues"
    
    if not url == expected:
        raise AssertionError()


def test_create_gitlab_url():
    url = ezissue.create_gitlab_url(1234)
    expected = "https://gitlab.com/api/v4/projects/1234/issues"
    
    if not url == expected:
        raise AssertionError()


def test_get_table_spec():
    l, thead = ezissue.get_table_spec('| asdfds | asdf | asdf |')
    
    if not l == 3:
        raise AssertionError()
    
    if not thead == ['asdfds', 'asdf', 'asdf']:
        raise AssertionError()


def test_create_issue_json():
    a = ezissue.create_issue_json(
        ['title', 'description', 'accept'],
        [[1], 2, 3],
        'github'
    )
    b = ezissue.create_issue_json(
        ['title', 'body', 'accept'],
        [[1], 2, 3],
        'gitlab'
    )



import os
import click
import getpass
import requests
import json as js

import ezissue.utils as u
from ezissue.converter.manipulation import *
from ezissue.secops.secops_basic import get_token
from ezissue.secops.secops_basic import folder_path
from ezissue.secops.secops_basic import write_tokens
from ezissue.secops.secops_basic import create_secure_key


GITHUB_BASE_URL = "https://api.github.com"
GITLAB_BASE_URL = "https://gitlab.com/api/v4"


def create_issue_json(configuration_row, values_row, repo_host):
    """
    Creates a python dict with the issue's values following the format
    dict({<table header>: <row value>}).
    """
    n_fields = len(configuration_row)
    d = dict()

    blacklist = ["acceptance criteria", "tasks"]

    if n_fields != len(values_row):
        u.error(
            'Error: markdown table header and contents columns do not match!')
        raise

    for idx in range(n_fields):
        configuration_row[idx] = configuration_row[idx].lower()

        if configuration_row[idx] == 'title':
            d.update({'title': values_row[idx]})

        elif configuration_row[idx] == 'description' and repo_host == 'github':
            d.update({'body': values_row[idx]})

        elif configuration_row[idx] == 'body' and repo_host == 'gitlab':
            d.update({'description': values_row[idx]})

        elif configuration_row[idx] in blacklist:
            if repo_host == 'gitlab':
                body = d['description']
                body = body + str(values_row[idx])
                d.update({'description': body})
            else:
                body = d['body']
                body = body + str(values_row[idx])
                d.update({'body': body})
        else:
            d.update({configuration_row[idx]: values_row[idx]})
    return d


def create_github_url(repo_name, owner):
    """
    Creates the github's issue endpoint URL for accessing the API.
    """
    github = "/repos/%s/%s/issues" % (owner.lower(), repo_name.lower())
    endpoint = GITHUB_BASE_URL + github
    return endpoint


def create_gitlab_url(repo_uid):
    """
    Creates the gitlab's issue endpoint URL for accessing the API.
    """
    gitlab = "/projects/%i/issues" % repo_uid
    endpoint = GITLAB_BASE_URL + gitlab
    return endpoint


def make_api_call(json_issue, url, host, debug):
    """
    Makes the API POST request for the issue creation.
    Returns the response object and the issue's dict.
    """
    u.debug(json_issue, debug)

    my_token = get_token(host)

    if not host == 'github':
        response = requests.post(
            url,
            data=js.dumps(json_issue),
            headers={
                'PRIVATE-TOKEN': my_token,
                'Content-Type': 'application/json'
            }
        )
    else:
        auth = 'Bearer %s' % my_token
        response = requests.post(
            url,
            data=js.dumps(json_issue),
            headers={
                'Accept': 'application/vnd.github.v3+json',
                'Authorization': auth,
                'Content-Type': 'application/json'
            }
        )
    return response, json_issue


@click.command()
@click.argument(
    "filename",
    required=True
)
@click.argument(
    "repo_host",
    type=click.Choice(["github", "gitlab"], case_sensitive=False),
    required=True
)
@click.option(
    "--subid",
    "-s",
    required=False,
    default="",
    type=str,
    help='Pass an sub-id into the issue\'s title'
)
@click.option(
    "--numerate",
    "-n",
    required=False,
    is_flag=True,
    help='Numerate the issues on the table\'s order'
)
@click.option(
    "--prefix",
    "-p",
    required=False,
    default="",
    type=click.Choice(["US", "TS", "", "BUG"], case_sensitive=False),
    help='Adds a prefix on the issue\'s title'
)
@click.option(
    "--debug",
    "-d",
    default=False,
    is_flag=True,
    help='Enables debug mode'
)
def main(filename, repo_host, prefix, subid, numerate, debug):
    """
    Main function.
    """
    if not os.path.isfile(folder_path + 'key.key'):
        config()

    try:
        file = open(filename)
        lines = get_all_lines(file)
        rows = []

        for idx, line in enumerate(lines):
            if idx == 0:
                col_count, columns = get_table_spec(line)
            elif idx == 1:
                pass
            else:
                rows.append(md_table_row_to_array(line))

        if repo_host == 'github':
            repo = u.get_from_user("Enter repo name: (Ex.: username/repo)")
            repo = repo.split('/')
            url = create_github_url(repo[1], repo[0])
        else:
            repo = int(u.get_from_user("Enter the repo id: (Ex.: 9120898)"))
            url = create_gitlab_url(repo)

        u.debug(repr(url), debug)
        u.debug(repr(repo_host), debug)

        for idx, row in enumerate(rows):
            row[0] = add_prefix_to_title(
                row[0], idx+1, prefix, subid, numerate)

        rows = make_md_formatting(columns, rows)

        for row in rows:
            response, issue = make_api_call(
                create_issue_json(columns, row, repo_host),
                url,
                repo_host,
                debug
            )
            u.show_resp_req(issue, response)

    finally:
        file.close()


def config():
    """
    Runs setup configuration on the CLI. Creates a hidden folder on the user's
    HOMEDIR with secure encrypted info.
    """
    u.notify("Config file not found! Initializing configuration...")

    ghtk = getpass.getpass(prompt="Please insert your github token: ")
    gltk = getpass.getpass(prompt="Please insert your gitlab token: ")

    create_secure_key()

    success = write_tokens(ghtk, gltk)
    if success:
        u.prompt("Created config files successfully!")
        u.prompt("(They're encrypted, don't worry)")
    else:
        u.prompt("Something went wrong, please try again.")


if __name__ == "__main__":
    main()



import os
import codecs
from os import path
from setuptools import setup
from setuptools import find_packages
from pip._internal.req import parse_requirements

here = path.abspath(path.dirname(__file__))

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allowes setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

# parse_requirements() returns generator of pip.req.InstallRequirement objects
INSTALL_REQS = parse_requirements('requirements.txt', session='hack')

# reqs is a list of requirements
REQUIREMENTS = [str(ir.req) for ir in INSTALL_REQS]

CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "Natural Language :: Portuguese",
    "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: Implementation :: CPython",
    "Programming Language :: Python :: Implementation :: PyPy",
    "Topic :: Software Development :: Libraries :: Python Modules",
]

setup(  # pragma: no cover
    name='ezissue-br',
    description="The ultimate aid in the issue creation process.",  # nopep8
    url='https://github.com/andre-filho/ezissue',
    long_description=codecs.open('README.md', 'rb', 'utf8').read(),
    long_description_content_type='text/markdown',
    author='Andre de Sousa Costa Filho',
    author_email='andre.filho001@outlook.com',
    version=codecs.open('VERSION.txt', 'rb', 'utf8').read(),
    packages=find_packages(),
    keywords=['ezissue-br', 'issue', 'markdown', 'api', 'github', 'gitlab', 'markdown to issue converter', 'pt-br'],
    entry_points={
        'console_scripts': [
            'brissue = ezissue.ezissue:main'
        ]
    },
    install_requires=REQUIREMENTS,
    license='GNU',
    classifiers=CLASSIFIERS,
)



from colored import fg
from colored import attr

RESET = attr(0)
NORMAL = fg(7)
DEBUG = fg(239)
NOTIFY = fg(14)
ERROR = fg(196)
SUC = fg(82)


def prompt(text):
    """
    Prints with the normal color.
    """
    print("%s%s %s" % (NORMAL, text, RESET))


def debug(text, show):
    """
    Prints a custom colored debug string.
    """
    if show:
        print("%s%s %s" % (DEBUG, text, RESET))


def notify(text):
    """
    Prints a custom colored notification string.
    """
    print("%s%s %s" % (NOTIFY, text, RESET))


def get_from_user(question):
    """
    Recieves a input with a colored question.
    """
    return input('%s%s %s\n' % (NORMAL, question, RESET))


def error(text):
    """
    Prints a custom colored error string.
    """
    print("%s%s %s" % (ERROR, text, RESET))


def show_resp_req(req_json, res):
    """
    Prints the request status with custom colors.
    """
    prompt('\nRequest\'s json ~> %s' % req_json)
    if res.status_code == 201:
        print('Request\'s response ~> %s%d%s' % (SUC, res.status_code, RESET))
    else:
        print('Request\'s response ~> %s%d%s' %
              (ERROR, res.status_code, RESET))



import re


def remove_md_titles(line):
    line = re.sub(
        r'((#\s)|(##\s)|(###\s)|(####\s)|(#####\s)|(######\s))',
        '',
        line
    )
    return line


def md_table_row_to_array(line):
    """
    Recieves a table line and parses it's content as a array of strings, with
    each element representing a column.
    """
    # replaces the | characters between the text with a totally random string
    line = re.sub(r'(\ \|\ )', '3Vb7adZJNY', line)
    line = re.sub(r'(\|\ )|(\ \|)', '', line)
    line = line.replace("\n", '')
    line = line.split('3Vb7adZJNY')
    return line


def add_md_checkbox(itemized_string):
    """
    Converts a itemized string in a formatted markdown checkbox list.
    """
    items = itemized_string.split(';')
    a = ""
    for item in items:
        a += str('- [ ] ' + item + '\n')
    return a


def format_description(string):
    """
    Adds the header and a final new line to the issue description string.
    """
    return str('**Descrição:**\n---\n\n' + string + '\n'*2)


def add_prefix_to_title(title, number, prefix, subid, numerate):
    """
    Formats the issue title to match the required for the project.
    """
    subid = subid.upper()
    prefix = prefix.upper()
    title = title.capitalize()

    if numerate:
        return str(prefix + subid + str(number) + " " + title)

    return str(prefix + subid + " " + title)


def get_all_lines(file):
    """
    Reads a file returning all it's lines in a array of lines format.
    """
    line = file.readline()
    lines = []

    while line:
        lines.append(line)
        line = file.readline()

    return lines


def get_table_spec(line):
    """
    Gets the table header and returns both it's length and it's formatted self.
    """
    thead = md_table_row_to_array(line)
    return len(thead), thead


def format_acc_criteria(string):
    """
    Formats the string adding the acceptance criteria header and adds a final
    new line.
    """
    checkboxes = add_md_checkbox(string)
    acc_criteria_title = "**Critérios de aceitação:**\n---\n\n"

    return "%s%s" % (acc_criteria_title, checkboxes)


def format_tasks(string):
    """
    Formats the string adding the tasks header and adds a final new line.
    """
    checkboxes = add_md_checkbox(string)
    tasks_title = "**Tarefas:**\n---\n\n"
    return "%s%s\n" % (tasks_title, checkboxes)


def make_md_formatting(configuration_header, content):
    """
    Dinamically formats the markdown content according to it's column header.
    """
    func_dict = {
        "description": format_description,
        "body": format_description,
        "acceptance criteria": format_acc_criteria,
        "tasks": add_md_checkbox,
    }

    cont_length = len(content)
    conf_header_length = len(configuration_header)

    for row in range(cont_length):
        for idx in range(conf_header_length):
            if len(configuration_header) == 0 or idx == 0:
                pass
            else:
                content[row][idx] = \
                    func_dict[configuration_header[idx].lower()](
                        str(content[row][idx]))
    return content
